import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Rectangle2D;
import java.util.Random;

import javax.swing.*;

import com.mindfusion.common.DateTime;
import com.mindfusion.common.Duration;
import com.mindfusion.diagramming.*;
import com.mindfusion.diagramming.Shape;


public class MainFrame extends JFrame
{
	DiagramView diagramView;

	void generateDiagram()
	{
		// adding items is slower when the diagram is shown inside a view
		// so we create a new diagram here
		Diagram diagram = new Diagram();
		diagram.setDefaultShape(Shape.fromId("Rectangle"));
		diagram.setShadowsStyle(ShadowsStyle.None);

		// adding a lot of items is faster when these properties are disabled
		diagram.setValidityChecks(false);
		diagram.setSelectAfterCreate(false);
		diagram.setAutoResize(AutoResize.None);
		diagram.setSelectionOnTop(false);

		int total = 10000;
		int rowSize = (int)Math.sqrt(total);
		int size = 20;
		int dist = 30;

		int diagSize = rowSize * dist;
		diagram.setBounds(
			new Rectangle2D.Float(
				0, 0, diagSize, diagSize)); 

		float x = (float)diagram.getBounds().getMinX();
		float y = (float)diagram.getBounds().getMinY();
		for (int i = 0; i < total; i++)
		{
			ShapeNode node = diagram.getFactory().createShapeNode(x, y, size, size);
			node.setText(Integer.toString(i));
			if (x > diagram.getBounds().getMinX())
			{
				diagram.getFactory().createDiagramLink(
					diagram.getNodes().get(i - 1), diagram.getNodes().get(i));
			}

			x += dist;
			if (x >= diagram.getBounds().getMaxX())
			{
				x = (float)diagram.getBounds().getMinX();
				y += dist;
			}
		}

		// index item positions for faster viewport clipping and hit testing
		diagram.setEnableSpatialIndex(true);

		// finally show the diagram inside the view
		diagramView.setDiagram(diagram);
	}

    private void initFrame()
    {
    	setTitle("MindFusion.Diagramming Sample: Stress Test");
        setSize(new Dimension(1200, 900));
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setLocationByPlatform(true);

    	diagramView = new DiagramView();

    	JScrollPane scrollPane = new JScrollPane(diagramView);

        JButton generateButton = new JButton("Create items");
        generateButton.setPreferredSize(new Dimension(150, 80));
        generateButton.addActionListener(new ActionListener() {
			@Override public void actionPerformed(ActionEvent e) {
				generateDiagram();
			}
		});


        JPanel spacer = new JPanel();
        spacer.setPreferredSize(new Dimension(10, 20));

    	JTextArea descriptionArea = new JTextArea();
        descriptionArea.setBackground(Color.white);
        descriptionArea.setText(
        	"Click the button to create 20000 nodes and links in total. Diagram\\'s spatial indexing allows " +
        	"for fast hit-test and viewport queries during rendering and user interactions.");
        descriptionArea.setEditable(false);
        descriptionArea.setAutoscrolls(false);
        descriptionArea.setLineWrap(true);
        descriptionArea.setWrapStyleWord(true);

		JPanel controlPanel = new JPanel();
		controlPanel.setLayout(
			new BoxLayout(controlPanel, BoxLayout.X_AXIS));
		controlPanel.add(generateButton);
		controlPanel.add(spacer);
		controlPanel.add(descriptionArea);

		for (Component child : controlPanel.getComponents())
		{
			if (child instanceof JComponent)
			{
				JComponent jchild = (JComponent)child;
				jchild.setAlignmentX(LEFT_ALIGNMENT);
			}
		}

        Container cp = this.getContentPane();
      	cp.setLayout(new BorderLayout());
      	cp.add(scrollPane, BorderLayout.CENTER);
		cp.add(controlPanel, BorderLayout.SOUTH);
    }

	public MainFrame()
	{
        initFrame();
    }

	private static final long serialVersionUID = 1L;
}
